@@ -1,8 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from common import *
-from datetime_strftime import *
-from domain_inversion import domain_inversion, eval_domain, localize_domain, \
+from .common import *
+from .datetime_strftime import *
+from .domain_inversion import domain_inversion, eval_domain, localize_domain, \
         merge, inverse_leaf, filter_leaf, concat, simplify, unique_value
-from environment import EvalEnvironment
-import timedelta
+from .environment import EvalEnvironment
+from . import timedelta
